#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#The MIT License (MIT)
#
#Copyright (c) <2013-2014> <Colin Duquesnoy and others, see AUTHORS.txt>
#
#Permission is hereby granted, free of charge, to any person obtaining a copy
#of this software and associated documentation files (the "Software"), to deal
#in the Software without restriction, including without limitation the rights
#to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
#copies of the Software, and to permit persons to whom the Software is
#furnished to do so, subject to the following conditions:
#
#The above copyright notice and this permission notice shall be included in
#all copies or substantial portions of the Software.
#
#THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
#IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
#FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
#AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
#LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
#OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
#THE SOFTWARE.
#
"""
This file contains all the pyqode-widgets QtDesigner plugins.
"""
# This only works with PyQt, PySide does not support the QtDesigner module
import os
if not 'QT_API' in os.environ:
    os.environ.setdefault("QT_API", "PyQt")
from pyqode.qt.QtGui import QIcon
import pyqode.widgets

PLUGINS_TYPES = {'QHomeWidget': pyqode.widgets.QHomeWidget,
                 'QInteractiveConsole': pyqode.widgets.QInteractiveConsole,
                 "QCodeEditTabWidget": pyqode.widgets.QCodeEditTabWidget,
                 "QPropertyGrid": pyqode.widgets.QPropertyGrid,
                 "QColorButton": pyqode.widgets.QColorButton,
                 "QTextStyleWidget": pyqode.widgets.QTextStyleWidget,
                 "QErrorsTable": pyqode.widgets.QErrorsTable}

try:
    from pyqode.core.plugins.pyqode_core_plugin import QCodeEditPlugin

    class QPythonCodeEditPlugin(QCodeEditPlugin):
        _module = 'pyqode.widgets'        # path to the widget's module
        _class = 'QHomeWidget'    # name of the widget class
        _name = "QHomeWidget"

        def createWidget(self, parent):
            return pyqode.widgets.QHomeWidget(parent=parent)

    class QInteractiveConsolePlugin(QCodeEditPlugin):
        _module = 'pyqode.widgets'        # path to the widget's module
        _class = 'QInteractiveConsole'    # name of the widget class
        _name = "QInteractiveConsole"

        def createWidget(self, parent):
            return pyqode.widgets.QInteractiveConsole(parent=parent)

    class QCodeEditTabWidgetPlugin(QCodeEditPlugin):
        _module = 'pyqode.widgets'        # path to the widget's module
        _class = 'QCodeEditTabWidget'    # name of the widget class
        _name = "QCodeEditTabWidget"

        def createWidget(self, parent):
            return pyqode.widgets.QCodeEditTabWidget(parent=parent)

    class QErrorsTablePlugin(QCodeEditPlugin):
        _module = 'pyqode.widgets'        # path to the widget's module
        _class = 'QErrorsTable'    # name of the widget class
        _name = "QErrorsTable"

        def createWidget(self, parent):
            return pyqode.widgets.QErrorsTable(parent=parent)

    class QPropertyGridPlugin(QCodeEditPlugin):
        _module = 'pyqode.widgets'        # path to the widget's module
        _class = 'QPropertyGrid'    # name of the widget class
        _name = "QPropertyGrid"

        def createWidget(self, parent):
            return pyqode.widgets.QPropertyGrid(parent=parent)

    class QColorButtonPlugin(QCodeEditPlugin):
        _module = 'pyqode.widgets'        # path to the widget's module
        _class = 'QColorButton'    # name of the widget class
        _name = "QColorButton"

        def createWidget(self, parent):
            return pyqode.widgets.QColorButton(parent=parent)

    class QTextStyleWidgetPlugin(QCodeEditPlugin):
        _module = 'pyqode.widgets'        # path to the widget's module
        _class = 'QTextStyleWidget'    # name of the widget class
        _name = "QTextStyleWidget"

        def createWidget(self, parent):
            return pyqode.widgets.QTextStyleWidget(parent=parent)


except ImportError:
    print("Cannot use pyQode plugins without PyQt4")



# -*- coding: utf-8 -*-
#
#The MIT License (MIT)
#
#Copyright (c) <2013-2014> <Colin Duquesnoy and others, see AUTHORS.txt>
#
#Permission is hereby granted, free of charge, to any person obtaining a copy
#of this software and associated documentation files (the "Software"), to deal
#in the Software without restriction, including without limitation the rights
#to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
#copies of the Software, and to permit persons to whom the Software is
#furnished to do so, subject to the following conditions:
#
#The above copyright notice and this permission notice shall be included in
#all copies or substantial portions of the Software.
#
#THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
#IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
#FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
#AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
#LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
#OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
#THE SOFTWARE.
#
"""
This module contains promoted widgets used in Qt Designer's ui.
"""
from pyqode.qt import QtGui, QtCore


class HomeListWidget(QtGui.QListWidget):
    """
    A custom list widget which keeps track of the mouse button that is
    pressed. It also changes the mouse cursor to a pointing hand cursor
    when the mouse cursor is over a list item.

    This class is used in QHomeWidget to ignore itemClicked when the right
    button is pressed.
    """
    def __init__(self, *args, **kwargs):
        QtGui.QListWidget.__init__(self, *args, **kwargs)
        self.setMouseTracking(True)

    def mousePressEvent(self, e):
        """
        Keeps track of the pressed button
        """
        self.mouseButton = e.button()
        QtGui.QListWidget.mousePressEvent(self, e)

    def mouseMoveEvent(self, e):
        """
        Display a pointing hand cursor when over an item. The cursor is
        reset to an ArrowCursor if there are no item under the cursor.
        """
        item = self.itemAt(e.pos())
        QtGui.QListWidget.mouseMoveEvent(self, e)
        self.setCursor(QtCore.Qt.PointingHandCursor if item else
            QtCore.Qt.ArrowCursor)




# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'dlg_unsaved_files.ui'
#
# Created: Wed Nov 13 10:27:38 2013
#      by: pyside-uic 0.2.14 running on PySide 1.1.2
#
# WARNING! All changes made in this file will be lost!

from pyqode.qt import QtCore, QtGui

class Ui_Dialog(object):
    def setupUi(self, Dialog):
        Dialog.setObjectName("Dialog")
        Dialog.resize(717, 301)
        self.verticalLayout = QtGui.QVBoxLayout(Dialog)
        self.verticalLayout.setObjectName("verticalLayout")
        self.label = QtGui.QLabel(Dialog)
        self.label.setObjectName("label")
        self.verticalLayout.addWidget(self.label)
        self.listWidget = QtGui.QListWidget(Dialog)
        self.listWidget.setSelectionMode(QtGui.QAbstractItemView.ExtendedSelection)
        self.listWidget.setObjectName("listWidget")
        self.verticalLayout.addWidget(self.listWidget)
        self.buttonBox = QtGui.QDialogButtonBox(Dialog)
        self.buttonBox.setOrientation(QtCore.Qt.Horizontal)
        self.buttonBox.setStandardButtons(QtGui.QDialogButtonBox.Cancel|QtGui.QDialogButtonBox.Discard|QtGui.QDialogButtonBox.SaveAll)
        self.buttonBox.setObjectName("buttonBox")
        self.verticalLayout.addWidget(self.buttonBox)

        self.retranslateUi(Dialog)
        QtCore.QObject.connect(self.buttonBox, QtCore.SIGNAL("accepted()"), Dialog.accept)
        QtCore.QObject.connect(self.buttonBox, QtCore.SIGNAL("rejected()"), Dialog.reject)
        QtCore.QMetaObject.connectSlotsByName(Dialog)

    def retranslateUi(self, Dialog):
        Dialog.setWindowTitle(QtGui.QApplication.translate("Dialog", "Dialog", None, QtGui.QApplication.UnicodeUTF8))
        self.label.setText(QtGui.QApplication.translate("Dialog", "The following files have unsaved changes:", None, QtGui.QApplication.UnicodeUTF8))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#The MIT License (MIT)
#
#Copyright (c) <2013-2014> <Colin Duquesnoy and others, see AUTHORS.txt>
#
#Permission is hereby granted, free of charge, to any person obtaining a copy
#of this software and associated documentation files (the "Software"), to deal
#in the Software without restriction, including without limitation the rights
#to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
#copies of the Software, and to permit persons to whom the Software is
#furnished to do so, subject to the following conditions:
#
#The above copyright notice and this permission notice shall be included in
#all copies or substantial portions of the Software.
#
#THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
#IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
#FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
#AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
#LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
#OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
#THE SOFTWARE.
#
"""
This setup script packages the widgets package of pyQode: pyqode.widgets
"""
from pyqode.widgets.errors_table import QErrorsTable
from pyqode.widgets.home import QHomeWidget
from pyqode.widgets.home import DarkColorScheme
from pyqode.widgets.home import ColorScheme
from pyqode.widgets.interactive import QInteractiveConsole
from pyqode.widgets.property_grid import QPropertyGrid
from pyqode.widgets.property_grid import QColorButton
from pyqode.widgets.property_grid import QTextStyleWidget
from pyqode.widgets.tabs import QCodeEditTabWidget


__version__ = "1.2.2"


__all__ = ["ColorScheme", "DarkColorScheme", "QCodeEditTabWidget",
           "QColorButton", "QErrorsTable", "QHomeWidget", "QInteractiveConsole",
           "QPropertyGrid", "QTextStyleWidget"]



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#The MIT License (MIT)
#
#Copyright (c) <2013-2014> <Colin Duquesnoy and others, see AUTHORS.txt>
#
#Permission is hereby granted, free of charge, to any person obtaining a copy
#of this software and associated documentation files (the "Software"), to deal
#in the Software without restriction, including without limitation the rights
#to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
#copies of the Software, and to permit persons to whom the Software is
#furnished to do so, subject to the following conditions:
#
#The above copyright notice and this permission notice shall be included in
#all copies or substantial portions of the Software.
#
#THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
#IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
#FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
#AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
#LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
#OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
#THE SOFTWARE.
#
"""
This module contains the implementation of a tab widget specialised to
show code editor tabs.
"""
import os
import sys
import pyqode.core
from pyqode.qt import QtCore, QtGui

# needed to build doc on readthedocs; should be fixed with the next version
# of pyqode.core
try:
    from pyqode.qt.QtGui import QDialog, QTabBar, QTabWidget
except:
    class QDialog(object):
        pass

    class QTabBar(object):
        pass

    class QTabWidget:
        pass

from pyqode.widgets.ui.dlg_unsaved_files_ui import Ui_Dialog


class DlgUnsavedFiles(QDialog, Ui_Dialog):
    def __init__(self, parent, files=None):
        if files is None:
            files = []
        QtGui.QDialog.__init__(self, parent)
        Ui_Dialog.__init__(self)
        self.setupUi(self)
        self.btSaveAll = self.buttonBox.button(QtGui.QDialogButtonBox.SaveAll)
        self.btSaveAll.clicked.connect(self.accept)
        self.discarded = False
        self.btDiscard = self.buttonBox.button(QtGui.QDialogButtonBox.Discard)
        self.btDiscard.clicked.connect(self.__setDiscared)
        self.btDiscard.clicked.connect(self.accept)
        for f in files:
            self.addFile(f)
        self.listWidget.itemSelectionChanged.connect(self.__onSelectionChanged)
        self.__onSelectionChanged()

    def addFile(self, filePath):
        icon = QtGui.QFileIconProvider().icon(QtCore.QFileInfo(filePath))
        item = QtGui.QListWidgetItem(icon, filePath)
        self.listWidget.addItem(item)

    def __setDiscared(self):
        self.discarded = True

    def __onSelectionChanged(self):
        nbItems = len(self.listWidget.selectedItems())
        if nbItems == 0:
            self.btSaveAll.setText("Save")
            self.btSaveAll.setEnabled(False)
        else:
            self.btSaveAll.setEnabled(True)
            self.btSaveAll.setText("Save selected")
            if nbItems == self.listWidget.count():
                self.btSaveAll.setText("Save all")


class ClosableTabBar(QTabBar):
    def __init__(self, parent):
        QtGui.QTabBar.__init__(self, parent)
        self.setTabsClosable(True)

    def mousePressEvent(self, qMouseEvent):
        QtGui.QTabBar.mousePressEvent(self, qMouseEvent)
        if qMouseEvent.button() == QtCore.Qt.MiddleButton:
            self.parentWidget().tabCloseRequested.emit(self.tabAt(
                qMouseEvent.pos()))


class QCodeEditTabWidget(QTabWidget):
    """
    Adds support for tab widgets based on any subclass of pyqode.core.QCodeEdit.

    It ensures that there is only one open editor tab for a specific filePath,
    it adds a few utility methods to quickly manipulate the current editor
    widget.

    It handles the tab close requests automatically and show a message box when
    a dirty tab widget is being closed. It also adds a convenience tabBar with a
    "close", "close others" and "close all" menu. (You can add custom actions by
    using the addAction and addSeparator methods).

    It exposes a variety of signal and slots for a better integration with
    your applications( dirtyChanged, saveCurrent, saveAll, closeAll,
    closeCurrent, closeOthers).
    """

    dirtyChanged = QtCore.Signal(bool)
    lastTabClosed = QtCore.Signal()
    tabClosed = QtCore.Signal(QtGui.QWidget)

    @property
    def currentEditor(self):
        """
        Returns the current editor widget or None if the current tab widget is
        not a subclass of QCodeEdit or if there is no open tab.
        """
        return self.__currentTab

    def __init__(self, parent):
        QtGui.QTabWidget.__init__(self, parent)
        self.__currentTab = None
        self.currentChanged.connect(self.__onCurrentChanged)
        self.__watcher = QtCore.QFileSystemWatcher()
        self.__watcher.fileChanged.connect(self.__onFileChanged)
        self.tabCloseRequested.connect(self.__onTabCloseRequested)
        tabBar = ClosableTabBar(self)
        tabBar.setContextMenuPolicy(QtCore.Qt.CustomContextMenu)
        tabBar.customContextMenuRequested.connect(self.__showTabContextMenu)
        self.setTabBar(tabBar)
        self.contextMenu = QtGui.QMenu()
        a = QtGui.QAction("Close", self)
        a.triggered.connect(self.close)
        self.contextMenu.addAction(a)
        a = QtGui.QAction("Close others", self)
        a.triggered.connect(self.closeOthers)
        self.addAction(a)
        a = QtGui.QAction("Close all", self)
        a.triggered.connect(self.closeAll)
        self.addAction(a)
        self._widgets = []  # keep a list of widgets (to avoid PyQt bug where
                            # the C++ class loose the wrapped obj type).

    @QtCore.Slot()
    def close(self):
        self.tabCloseRequested.emit(self.currentIndex())

    @QtCore.Slot()
    def closeOthers(self):
        cw = self.currentWidget()
        self.tryCloseDirtyTabs(exept=cw)
        i = 0
        while self.count() > 1:
            w = self._widgets[i]
            if w != cw:
                self.removeTab(i)
            else:
                i = 1

    @QtCore.Slot()
    def closeAll(self):
        if self.tryCloseDirtyTabs():
            while self.count():
                widget = self._widgets[0]
                self.removeTab(0)
                self.tabClosed.emit(widget)
            return True
        return False

    @QtCore.Slot()
    def saveCurrent(self, filePath=None, force=False):
        """
        Save current tab contend. Specify filePath and set force to true to save as.
        """
        try:
            self.__currentTab.saveToFile(filePath, force)
            # adapt tab title on save as
            if filePath:
                self.setTabText(self.currentIndex(),
                                QtCore.QFileInfo(filePath).fileName())
            return True
        except AttributeError:  # not an editor widget
            pass
        return False

    @QtCore.Slot()
    def saveAll(self, filePath=None):
        for i in range(self.count()):
            try:
                self._widgets[i].saveToFile(filePath)
                self.setTabText(i, self._widgets[i].fileName)
            except AttributeError:
                pass

    def addAction(self, action):
        self.contextMenu.addAction(action)

    def addSeparator(self):
        return self.contextMenu.addSeparator()

    def isFileOpen(self, filePath):
        """
        Checks if the filePath is already open in an editor tab.

        :returns: The tab index if found or -1
        """
        for i, widget in enumerate(self._widgets):
            try:
                if widget.filePath == filePath:
                    return i
            except AttributeError:
                pass  # not an editor widget
        return -1

    def addEditorTab(self, qCodeEdit, icon=None):
        """
        Adds an editor tab widget, set its text as the editor.fileName and
        set it as the active tab.

        The widget is only added if there is no other editor tab open with the
        same filename, else the already open tab is set as current.

        :param qCodeEdit: The code editor widget tab to add
        :type qCodeEdit: pyqode.core.QCodeEdit

        :param icon: The tab widget icon. Optional
        :type icon: QtGui.QIcon or None
        """
        index = self.isFileOpen(qCodeEdit.filePath)
        if index != -1:
            # already open, just show it
            self.setCurrentIndex(index)
            # no need to keep this instance
            qCodeEdit.deleteLater()
            del qCodeEdit
            return
        if not icon:
            icon = QtGui.QFileIconProvider().icon(
                QtCore.QFileInfo(qCodeEdit.filePath))
        name = qCodeEdit.fileName
        if self.hasSameName(name):
            name = self.renameDuplicateTabs(name, qCodeEdit.filePath)
        qCodeEdit._tab_name = name
        index = self.addTab(qCodeEdit, icon, name)
        self.setCurrentIndex(index)
        self.setTabText(index, name)
        qCodeEdit.setFocus(True)
        if hasattr(qCodeEdit, "fileWatcherMode"):
            qCodeEdit.fileWatcherMode.fileDeleted.connect(self._onFileDeleted)

    def addTab(self, elem, icon, name):
        self._widgets.append(elem)
        return super().addTab(elem, icon, name)

    def hasSameName(self, name):
        for i in range(self.count()):
            if self.tabText(i) == name:
                return True
        return False

    def renameDuplicateTabs(self, name, path):
        for i in range(self.count()):
            if self.tabText(i) == name:
                filePath = self._widgets[i].filePath
                parentDir = os.path.split(os.path.abspath(
                    os.path.join(filePath, os.pardir)))[1]
                new_name = os.path.join(parentDir, name)
                self.setTabText(i, new_name)
                self._widgets[i]._tab_name = new_name
                break
        parentDir = os.path.split(os.path.abspath(
            os.path.join(path, os.pardir)))[1]
        return os.path.join(parentDir, name)

    def resetSettings(self, settings):
        """
        Resets the settings of all opened editor
        """
        for widget in self._widgets:
            try:
                widget.settings = settings
            except AttributeError:
                pass

    def resetStyle(self, style):
        """
        Resets the style of all opened editor
        """
        for widget in self._widgets:
            try:
                widget.style = style
                widget.repaint()
                widget.syntaxHighlighterMode.rehighlight()
            except AttributeError:
                pass

    def refreshIcons(self, useTheme=True):
        for widget in self._widgets:
            try:
                widget.refreshIcons(useTheme=useTheme)
            except AttributeError:
                pass

    def __onCurrentChanged(self, index):
        widget = self._widgets[index]
        if widget == self.__currentTab:
            return
        try:
            if self.__currentTab:
                self.__currentTab.dirtyChanged.disconnect(self.__onDirtyChanged)
        except AttributeError:
            pass  # not an editor widget
        self.__currentTab = widget
        try:
            if self.__currentTab:
                self.__currentTab.dirtyChanged.connect(self.__onDirtyChanged)
                self.__onDirtyChanged(self.__currentTab.dirty)
                self.__currentTab.setFocus()
        except AttributeError:
            pass  # not an editor widget

    def __onFileChanged(self, filePath):
        # check if this is a real external change, we actually got called when
        # the user save the file from the application
        # add it to the dialog list, the dialog will be show when the editor
        # will get focus again.
        pass

    def removeTab(self, p_int):
        QTabWidget.removeTab(self, p_int)
        widget = self._widgets.pop(p_int)
        if widget == self.__currentTab:
            if self.__currentTab:
                self.__currentTab.dirtyChanged.disconnect(self.__onDirtyChanged)
            self.__currentTab = None
        self.tabClosed.emit(widget)
        widget.deleteLater()
        del widget

    def __onTabCloseRequested(self, index):
        widget = self._widgets[index]
        try:
            if not widget.dirty:
                self.removeTab(index)
            else:
                dlg = DlgUnsavedFiles(self, files=[widget.filePath])
                if dlg.exec_() == dlg.Accepted:
                    if not dlg.discarded:
                        widget.saveToFile()
                    self.removeTab(index)
        except AttributeError as e:
            pyqode.core.logger.warning(e)
            pass  # do nothing, let the user handle this case
        if self.count() == 0:
            self.lastTabClosed.emit()

    def __showTabContextMenu(self, position):
        self.contextMenu.popup(self.mapToGlobal(position))

    def tryCloseDirtyTabs(self, exept=None):
        widgets, filenames = self.__collectDirtyWidgets(exept=exept)
        if not len(filenames):
            return True
        dlg = DlgUnsavedFiles(self, files=filenames)
        if dlg.exec_() == dlg.Accepted:
            if not dlg.discarded:
                for item in dlg.listWidget.selectedItems():
                    filename = item.text()
                    for w in widgets:
                        if w.filePath == filename:
                            break
                    if w != exept:
                        w.saveToFile()
                        self.removeTab(self.indexOf(w))
            return True
        return False

    def __collectDirtyWidgets(self, exept=None):
        widgets = []
        filenames = []
        for i in range(self.count()):
            widget = self._widgets[i]
            try:
                if widget.dirty and widget != exept:
                    widgets.append(widget)
                    filenames.append(widget.filePath)
            except AttributeError:
                pass
        return widgets, filenames

    def __onDirtyChanged(self, dirty):
        try:
            title = self.currentWidget()._tab_name
            if dirty:
                self.setTabText(self.currentIndex(), "* " + title)
            else:
                self.setTabText(self.currentIndex(), title)
        except AttributeError:
            pass
        self.dirtyChanged.emit(dirty)

    def closeEvent(self, event):
        if not self.tryCloseDirtyTabs():
            event.ignore()
        else:
            event.accept()

    def _onFileDeleted(self, editor):
        self.removeTab(self.indexOf(editor))


if __name__ == "__main__":

    def main():
        app = QtGui.QApplication(sys.argv)
        tw = QCodeEditTabWidget(None)
        e = pyqode.core.QGenericCodeEdit(tw)
        e.openFile(__file__)
        tw.addEditorTab(e)

        e = pyqode.core.QGenericCodeEdit(tw)
        e.openFile(pyqode.core.__file__)
        tw.addEditorTab(e)

        e = pyqode.core.QGenericCodeEdit(tw)
        e.openFile(pyqode.widgets.__file__)
        tw.addEditorTab(e)

        tw.showMaximized()
        app.exec_()

    main()



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'home.ui'
#
# Created: Wed Nov 13 10:27:38 2013
#      by: pyside-uic 0.2.14 running on PySide 1.1.2
#
# WARNING! All changes made in this file will be lost!

from pyqode.qt import QtCore, QtGui

class Ui_Form(object):
    def setupUi(self, Form):
        Form.setObjectName("Form")
        Form.resize(719, 500)
        self.gridLayout = QtGui.QGridLayout(Form)
        self.gridLayout.setContentsMargins(0, 0, 0, 0)
        self.gridLayout.setSpacing(0)
        self.gridLayout.setObjectName("gridLayout")
        self.frame = QtGui.QFrame(Form)
        self.frame.setMinimumSize(QtCore.QSize(0, 0))
        self.frame.setFrameShape(QtGui.QFrame.StyledPanel)
        self.frame.setFrameShadow(QtGui.QFrame.Raised)
        self.frame.setObjectName("frame")
        self.verticalLayout = QtGui.QVBoxLayout(self.frame)
        self.verticalLayout.setSpacing(0)
        self.verticalLayout.setContentsMargins(0, 0, 0, 0)
        self.verticalLayout.setObjectName("verticalLayout")
        self.horizontalLayoutHeader = QtGui.QHBoxLayout()
        self.horizontalLayoutHeader.setSpacing(0)
        self.horizontalLayoutHeader.setContentsMargins(-1, 0, -1, -1)
        self.horizontalLayoutHeader.setObjectName("horizontalLayoutHeader")
        self.lblIcon = QtGui.QLabel(self.frame)
        self.lblIcon.setText("")
        self.lblIcon.setObjectName("lblIcon")
        self.horizontalLayoutHeader.addWidget(self.lblIcon)
        self.lblTitle = QtGui.QLabel(self.frame)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Expanding, QtGui.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.lblTitle.sizePolicy().hasHeightForWidth())
        self.lblTitle.setSizePolicy(sizePolicy)
        self.lblTitle.setStyleSheet("font: 20pt \"Verdana\";")
        self.lblTitle.setObjectName("lblTitle")
        self.horizontalLayoutHeader.addWidget(self.lblTitle)
        self.verticalLayout.addLayout(self.horizontalLayoutHeader)
        self.horizontalLayoutMain = QtGui.QHBoxLayout()
        self.horizontalLayoutMain.setSpacing(0)
        self.horizontalLayoutMain.setContentsMargins(9, 9, 9, 9)
        self.horizontalLayoutMain.setObjectName("horizontalLayoutMain")
        self.frameRecents = QtGui.QFrame(self.frame)
        self.frameRecents.setFrameShape(QtGui.QFrame.StyledPanel)
        self.frameRecents.setFrameShadow(QtGui.QFrame.Raised)
        self.frameRecents.setObjectName("frameRecents")
        self.verticalLayout_4 = QtGui.QVBoxLayout(self.frameRecents)
        self.verticalLayout_4.setSpacing(0)
        self.verticalLayout_4.setContentsMargins(0, 0, 0, 0)
        self.verticalLayout_4.setObjectName("verticalLayout_4")
        self.verticalLayout_2 = QtGui.QVBoxLayout()
        self.verticalLayout_2.setSpacing(0)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.lblRecents = QtGui.QLabel(self.frameRecents)
        self.lblRecents.setFrameShadow(QtGui.QFrame.Sunken)
        self.lblRecents.setAlignment(QtCore.Qt.AlignCenter)
        self.lblRecents.setObjectName("lblRecents")
        self.verticalLayout_2.addWidget(self.lblRecents)
        self.lwRecents = HomeListWidget(self.frameRecents)
        sizePolicy = QtGui.QSizePolicy(QtGui.QSizePolicy.Fixed, QtGui.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.lwRecents.sizePolicy().hasHeightForWidth())
        self.lwRecents.setSizePolicy(sizePolicy)
        self.lwRecents.setMaximumSize(QtCore.QSize(16777215, 16777215))
        self.lwRecents.setProperty("cursor", QtCore.Qt.ArrowCursor)
        self.lwRecents.setEditTriggers(QtGui.QAbstractItemView.NoEditTriggers)
        self.lwRecents.setProperty("showDropIndicator", False)
        self.lwRecents.setObjectName("lwRecents")
        self.verticalLayout_2.addWidget(self.lwRecents)
        self.verticalLayout_4.addLayout(self.verticalLayout_2)
        self.horizontalLayoutMain.addWidget(self.frameRecents)
        spacerItem = QtGui.QSpacerItem(20, 20, QtGui.QSizePolicy.Fixed, QtGui.QSizePolicy.Minimum)
        self.horizontalLayoutMain.addItem(spacerItem)
        self.framequickStart = QtGui.QFrame(self.frame)
        self.framequickStart.setFrameShape(QtGui.QFrame.StyledPanel)
        self.framequickStart.setFrameShadow(QtGui.QFrame.Raised)
        self.framequickStart.setObjectName("framequickStart")
        self.verticalLayout_5 = QtGui.QVBoxLayout(self.framequickStart)
        self.verticalLayout_5.setSpacing(0)
        self.verticalLayout_5.setContentsMargins(0, 0, 0, 0)
        self.verticalLayout_5.setObjectName("verticalLayout_5")
        self.verticalLayoutQuickStart = QtGui.QVBoxLayout()
        self.verticalLayoutQuickStart.setSpacing(0)
        self.verticalLayoutQuickStart.setObjectName("verticalLayoutQuickStart")
        self.lblQuickStart = QtGui.QLabel(self.framequickStart)
        self.lblQuickStart.setFrameShadow(QtGui.QFrame.Sunken)
        self.lblQuickStart.setAlignment(QtCore.Qt.AlignCenter)
        self.lblQuickStart.setObjectName("lblQuickStart")
        self.verticalLayoutQuickStart.addWidget(self.lblQuickStart)
        self.lwQuickStart = HomeListWidget(self.framequickStart)
        self.lwQuickStart.setProperty("cursor", QtCore.Qt.ArrowCursor)
        self.lwQuickStart.setObjectName("lwQuickStart")
        self.verticalLayoutQuickStart.addWidget(self.lwQuickStart)
        self.verticalLayout_5.addLayout(self.verticalLayoutQuickStart)
        self.horizontalLayoutMain.addWidget(self.framequickStart)
        self.verticalLayout.addLayout(self.horizontalLayoutMain)
        self.gridLayout.addWidget(self.frame, 0, 0, 1, 1)

        self.retranslateUi(Form)
        QtCore.QMetaObject.connectSlotsByName(Form)

    def retranslateUi(self, Form):
        Form.setWindowTitle(QtGui.QApplication.translate("Form", "Form", None, QtGui.QApplication.UnicodeUTF8))
        self.lblTitle.setText(QtGui.QApplication.translate("Form", "<html><head/><body><p><span style=\" font-size:20pt;\">Welcome to %s</span></p></body></html>", None, QtGui.QApplication.UnicodeUTF8))
        self.lblRecents.setText(QtGui.QApplication.translate("Form", "Recents", None, QtGui.QApplication.UnicodeUTF8))
        self.lblQuickStart.setText(QtGui.QApplication.translate("Form", "Quick start Actions", None, QtGui.QApplication.UnicodeUTF8))

from pyqode.widgets._promoted import HomeListWidget


#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#The MIT License (MIT)
#
#Copyright (c) <2013-2014> <Colin Duquesnoy and others, see AUTHORS.txt>
#
#Permission is hereby granted, free of charge, to any person obtaining a copy
#of this software and associated documentation files (the "Software"), to deal
#in the Software without restriction, including without limitation the rights
#to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
#copies of the Software, and to permit persons to whom the Software is
#furnished to do so, subject to the following conditions:
#
#The above copyright notice and this permission notice shall be included in
#all copies or substantial portions of the Software.
#
#THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
#IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
#FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
#AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
#LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
#OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
#THE SOFTWARE.
#
"""
Contains a custom QTableWidget for easier displaying of CheckerMessages
"""
import pyqode.core
from pyqode.core.system import memoized
from pyqode.qt import QtCore, QtGui

# needed to build doc on readthedocs; should be fixed with the next version
# of pyqode.core
try:
    from pyqode.qt.QtGui import QTableWidget
except:
    class QTableWidget(object):
        pass

COL_TYPE = 1
COL_FILE_NAME = 2
COL_LINE_NBR = 3
COL_MSG = 4
COL_PATH = 5


class QErrorsTable(QTableWidget):
    """
    Extends a QtGui.QTableWidget to easily show
    :class:`pyqode.core.CheckerMessage`.

    You add messages to the table using
    :meth:`pyqode.python.QErrorsTable.addMessage`. You clear the table using
    :meth:`pyqode.python.QErrorsTable.clear`
    """
    #: Signal emitted when a message is activated, the clicked signal is passed
    #: as a parameter
    messageActivated = QtCore.Signal(pyqode.core.CheckerMessage)

    def __init__(self, parent=None):
        QtGui.QTableWidget.__init__(self, parent)
        self.setColumnCount(6)
        self.setHorizontalHeaderLabels(
            ["Nr", "Type", "File name", "Line", "Description", "File path"])
        #self.horizontalHeader().showSortIndicator = True
        self.horizontalHeader().setResizeMode(QtGui.QHeaderView.ResizeToContents)
        self.horizontalHeader().setResizeMode(COL_MSG, QtGui.QHeaderView.Stretch)
        self.setMinimumSize(900, 200)
        self.itemActivated.connect(self._onItemActivated)
        self.setSelectionMode(self.SingleSelection)
        self.setSelectionBehavior(self.SelectRows)
        self.setContextMenuPolicy(QtCore.Qt.CustomContextMenu)
        self.customContextMenuRequested.connect(self.showContextMenu)
        self.contextMenu = QtGui.QMenu()
        self.actionCopyDescription = QtGui.QAction("Copy", self)
        self.actionCopyDescription.triggered.connect(
            self.copyCellText)
        self.contextMenu.addAction(self.actionCopyDescription)

    def copyCellText(self):
        """
        Copies the description of the selected message to the clipboard
        """
        txt = self.currentItem().text()
        QtGui.QApplication.clipboard().setText(txt)

    def showContextMenu(self, pos):
        self.contextMenu.exec_(self.mapToGlobal(pos))

    def clear(self, *args, **kwargs):
        """
        Clears the tables and the message list
        """
        QtGui.QTableWidget.clear(self, *args, **kwargs)
        self.setRowCount(0)
        self.setColumnCount(6)
        self.setHorizontalHeaderLabels(
            ["Nr", "Type", "File name", "Line", "Description", "File path"])

    @memoized
    def makeIcon(self, icon):
        if isinstance(icon, tuple):
            return QtGui.QIcon.fromTheme(
                icon[0], QtGui.QIcon(icon[1]))
        elif isinstance(icon, str):
            return QtGui.QIcon(icon)
        else:
            return None

    def addMessage(self, checkerMessage):
        """
        Adds a checker message to the table.

        :param checkerMessage: The message to add
        :type checkerMessage: pyqode.core.CheckerMessage
        """
        #self._messages.append(checkerMessage)
        row = self.rowCount()
        self.insertRow(row)

        # Nr
        item = QtGui.QTableWidgetItem(str(row + 1).zfill(3))
        item.setFlags(QtCore.Qt.ItemIsEnabled | QtCore.Qt.ItemIsSelectable)
        item.setData(QtCore.Qt.UserRole, checkerMessage)
        self.setItem(row, 0, item)

        # type
        item = QtGui.QTableWidgetItem(self.makeIcon(checkerMessage.icon),
                                      checkerMessage.statusString)
        item.setFlags(QtCore.Qt.ItemIsEnabled | QtCore.Qt.ItemIsSelectable)
        item.setData(QtCore.Qt.UserRole, checkerMessage)
        self.setItem(row, COL_TYPE, item)

        # filename
        item = QtGui.QTableWidgetItem(
            QtCore.QFileInfo(checkerMessage.filename).fileName())
        item.setFlags(QtCore.Qt.ItemIsEnabled | QtCore.Qt.ItemIsSelectable)
        item.setData(QtCore.Qt.UserRole, checkerMessage)
        self.setItem(row, COL_FILE_NAME, item)

        # line
        if checkerMessage.line <= 0:
            item = QtGui.QTableWidgetItem("----")
        else:
            item = QtGui.QTableWidgetItem(str(checkerMessage.line).zfill(4))
        item.setFlags(QtCore.Qt.ItemIsEnabled | QtCore.Qt.ItemIsSelectable)
        item.setData(QtCore.Qt.UserRole, checkerMessage)
        self.setItem(row, COL_LINE_NBR, item)

        # desc
        item = QtGui.QTableWidgetItem(checkerMessage.description)
        item.setFlags(QtCore.Qt.ItemIsEnabled | QtCore.Qt.ItemIsSelectable)
        item.setData(QtCore.Qt.UserRole, checkerMessage)
        self.setItem(row, COL_MSG, item)

        # filename
        item = QtGui.QTableWidgetItem(checkerMessage.filename)
        item.setFlags(QtCore.Qt.ItemIsEnabled | QtCore.Qt.ItemIsSelectable)
        item.setData(QtCore.Qt.UserRole, checkerMessage)
        self.setItem(row, COL_PATH, item)

    def _onItemActivated(self, item):
        """
        Emits the message activated signal
        """
        msg = item.data(QtCore.Qt.UserRole)
        self.messageActivated.emit(msg)


if __name__ == "__main__":
    import sys

    def printMessage(msg):
        print("Message activated: %s" % msg)

    def main():
        app = QtGui.QApplication(sys.argv)
        t = QErrorsTable()
        t.messageActivated.connect(printMessage)

        t.addMessage(pyqode.core.CheckerMessage(
            "A warning message", pyqode.core.MSG_STATUS_WARNING, 4,
            filename="/home/usr/file.py"))

        t.addMessage(pyqode.core.CheckerMessage(
            "An error message", pyqode.core.MSG_STATUS_ERROR, 569,
            filename="/home/usr/file2.py"))

        t.addMessage(pyqode.core.CheckerMessage(
            "An info message", pyqode.core.MSG_STATUS_INFO, 169,
            filename="/home/usr/file3.py"))

        t.addMessage(pyqode.core.CheckerMessage(
            "Tata", pyqode.core.MSG_STATUS_WARNING, 4,
            filename="/home/usr/file3.py"))

        t.addMessage(pyqode.core.CheckerMessage(
            "Titi", pyqode.core.MSG_STATUS_ERROR, 4,
            filename="/home/usr/file2.py"))

        t.addMessage(pyqode.core.CheckerMessage(
            "Toto", pyqode.core.MSG_STATUS_INFO, 4,
            filename="/home/usr/file1.py"))
        t.setSortingEnabled(True)
        t.show()
        app.exec_()

    main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#The MIT License (MIT)
#
#Copyright (c) <2013-2014> <Colin Duquesnoy and others, see AUTHORS.txt>
#
#Permission is hereby granted, free of charge, to any person obtaining a copy
#of this software and associated documentation files (the "Software"), to deal
#in the Software without restriction, including without limitation the rights
#to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
#copies of the Software, and to permit persons to whom the Software is
#furnished to do so, subject to the following conditions:
#
#The above copyright notice and this permission notice shall be included in
#all copies or substantial portions of the Software.
#
#THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
#IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
#FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
#AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
#LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
#OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
#THE SOFTWARE.
#
"""
This module contains interactive widgets:
    - interactive console: a text edit made to run subprocesses interactively
"""
import logging
import os
import sys

# needed to build doc on readthedocs; should be fixed with the next version
# of pyqode.core
try:
    from pyqode.qt.QtCore import Qt, Signal, Property, QProcess
    from pyqode.qt.QtGui import QTextEdit, QApplication, QTextCursor, QColor
except:
    class Property(object):
        def __init__(self, *args, **kwargs):
            pass

    class QProcess(object):
        pass


    class Foo(object):
        pass


    class QTextEdit(object):
        ExtraSelection = Foo


    class QColor(object):
        def __init__(self, *args, **kwargs):
            pass


class QInteractiveConsole(QTextEdit):
    """
    An interactive console is a QTextEdit specialised to run an interactive
    process. The user will see the process outputs and will be able to
    interact with the process by typing inputs.

    You can customize the colors using the following attributes:
        - processOutputColor:
        - userInputColor: color of the user inputs. Green by default
        - appMessageColor: a color for custom application message (
                           process started, process finished)
        - backgroundColor: the console background color
    """
    #: Signal emitted when the process has finished.
    processFinished = Signal(int)

    @property
    def mergeStdErrWithStdOut(self):
        """
        Merge stderr with stdout. Default is False. If set to true, stderr and
        stdin won't have distinctive colors, i.e. stderr output will be display
        with the same color as stdout.
        """
        return self._mergeOutput

    @mergeStdErrWithStdOut.setter
    def mergeStdErrWithStdOut(self, value):
        self._mergeOutput = value
        if value:
            self.process.setProcessChannelMode(QProcess.MergedChannels)
        else:
            self.process.setProcessChannelMode(QProcess.SeparateChannels)

    def __init__(self, parent=None,
                 writer=None,
                 mergeStdErrWithStdOut=False,
                 clearOnStart=True,
                 errorColor=QColor("#FF0000"),
                 processOutputColor=QColor("#404040"),
                 userInputColor=QColor("#22AA22"),
                 appMessageColor=QColor("#4040FF")):
        QTextEdit.__init__(self, parent)
        self._processOutputColor = processOutputColor
        self._appMessageColor = appMessageColor
        self._userInputColor = userInputColor
        self._errorColor = errorColor
        self._usrBuffer = ""
        self._clearOnStart = clearOnStart
        self.process = QProcess()
        self._mergeOutput = None
        self.mergeStdErrWithStdOut = mergeStdErrWithStdOut
        self.process.finished.connect(self._writeProcessFinished)
        #self.process.started.connect(self._writeProcessStarted)
        self.process.error.connect(self._writeProcessErrored)
        self.process.readyReadStandardError.connect(self._onOutputReady)
        self.process.readyReadStandardOutput.connect(self._onErrorReady)
        self._running = False
        if not writer:
            self._writer = self.write
        else:
            self._writer = writer

    def setWriter(self, writer):
        """
        Changes the write function. A writer function must have the following
        prototype:

        .. code-block:: python

            def write(textEdit, text, color)
        """
        if self._writer != writer and self._writer:
            self._writer = None
        if writer:
            self._writer = writer

    def _onOutputReady(self):
        raw = self.process.readAllStandardError()
        if sys.version_info[0] == 2:
            self._writer(self, unicode(raw, "utf-8"), self._errorColor)
        else:
            self._writer(self, str(raw, "utf-8"), self._errorColor)

    def _onErrorReady(self):
        raw = self.process.readAllStandardOutput()
        if sys.version_info[0] == 2:
            self._writer(self, unicode(raw, "utf-8"), self._processOutputColor)
        else:
            self._writer(self, str(raw, "utf-8"), self._processOutputColor)

    def __getBackgroundColor(self):
        p = self.palette()
        return p.color(p.Base)

    def __setBackgroundColor(self, color):
        p = self.palette()
        p.setColor(p.Base, color)
        p.setColor(p.Text, self.processOutputColor)
        self.setPalette(p)

    #: The console background color. Default is white.
    backgroundColor = Property(
        QColor, __getBackgroundColor, __setBackgroundColor,
        "The console background color")

    def __getProcessOutputColor(self):
        return self._processOutputColor

    def __setProcessOutputColor(self, color):
        self._processOutputColor = color
        p = self.palette()
        p.setColor(p.Text, self.processOutputColor)
        self.setPalette(p)

    #: Color of the process output. Default is black.
    processOutputColor = Property(
        QColor, __getProcessOutputColor, __setProcessOutputColor,
        doc="The color of the process output (stdout)")

    def __getErrorColor(self):
        return self._errorColor

    def __setErrorColor(self, color):
        self._errorColor = color

    #: Color for stderr output if
    # :attr:`pyqode.widgets.QInteractiveConsole.mergeStderrWithStdout`is False.
    errorColor = Property(
        QColor, __getProcessOutputColor, __setProcessOutputColor,
        doc="The color of the error messages (stderr)")

    def __getUserInputOutputColor(self):
        return self._userInputColor

    def __setUserInputOutputColor(self, color):
        self._userInputColor = color

    #: Color for user inputs. Default is green.
    usrInputColor = Property(
        QColor, __getUserInputOutputColor, __setUserInputOutputColor,
        doc="The color of the user inputs")

    def __getAppMessageColor(self):
        return self._appMessageColor

    def __setAppMessageColor(self, color):
        self._appMessageColor = color

    #: Application messages color, e.g. process started/finished messages.
    #: Default is blue.
    appMessageColor = Property(
        QColor, __getAppMessageColor, __setAppMessageColor,
        doc="The color of the application messages (such as process "
            "informations)")

    def closeEvent(self, *args, **kwargs):
        if self.process.state() == QProcess.Running:
            self.process.terminate()

    def runProcess(self, process, args=[], cwd=None):
        """
        Run a process interactively.

        E.g:

        .. code-block:: python

            ic = pyqode.python.QInteractiveConsole()
            ic.runProcess(sys.executable, ["setup.py", "install"],
                          cwd="/home/myUserName/myProj")

        :param process: Process to run
        :type process: str

        :param args: List of arguments (list of str)
        :type args: list

        :param cwd: Working directory
        :type cwd: str
        """
        if not self._running:
            if cwd:
                self.process.setWorkingDirectory(cwd)
            self._running = True
            self._process = process
            self._args = args
            if self._clearOnStart:
                self.clear()
            self.process.start(process, args)
            self._writeProcessStarted()
        else:
            logging.getLogger("pyqode").warning(
                "QInteractiveConsole: Process already running")

    def keyPressEvent(self, QKeyEvent):
        if QKeyEvent.key() == Qt.Key_Return or QKeyEvent.key() == Qt.Key_Enter:
            # send the user input to the child process
            self._usrBuffer += "\n"
            if sys.version_info[0] == 3:
                self.process.write(bytes(self._usrBuffer, "utf-8"))
            self.process.write(self._usrBuffer.encode("utf-8"))
            self._usrBuffer = ""
        else:
            if QKeyEvent.key() != Qt.Key_Backspace:
                txt = QKeyEvent.text()
                self._usrBuffer += txt
                self.setTextColor(self._userInputColor)
            else:
                self._usrBuffer = self._usrBuffer[0:len(self._usrBuffer)-1]
        # text is inserted here, the text color must be defined before this line
        QTextEdit.keyPressEvent(self, QKeyEvent)

    def _writeProcessFinished(self, exitCode, exitStatus):
        self._writer(self, "\nProcess finished with exit code %d" % exitCode,
                     self._appMessageColor)
        self._running = False
        self.processFinished.emit(exitCode)

    def _writeProcessStarted(self):
        self._writer(self, "{0} {1}\n".format(self._process, " ".join(self._args)),
                     self._appMessageColor)
        self._running = True

    def _writeProcessErrored(self):
        self._writer(self, "Failed to start {0} {1}\n".format(
            self._process, " ".join(self._args)), self._errorColor)
        self._running = False

    @staticmethod
    def write(textEdit, text, color):
        """
        Default write function. Move the cursor to the cursor to the end and
        insert text with the specified color.

        :param textEdit: QInteractiveConsole instance
        :type textEdit: pyqode.widgets.QInteractiveConsole

        :param text: Text to write
        :type text: str

        :param color: Desired text color
        :type color: QColor
        """
        textEdit.moveCursor(QTextCursor.End)
        textEdit.setTextColor(color)
        textEdit.insertPlainText(text)
        textEdit.moveCursor(QTextCursor.End)


if __name__ == "__main__":
    if len(sys.argv) <= 1:
        app = QApplication(sys.argv)
        t = QInteractiveConsole(mergeStdErrWithStdOut=False)
        t.resize(800, 600)
        t.show()
        t.runProcess(sys.executable, args=[__file__, "subprocess"], cwd=os.getcwd())
        #t.runProcess("sh", ["-i"])
        app.exec_()
    else:
        print("Hello from a subprocess!")
        if sys.version_info[0] >= 3:
            d = input("Enter something:\n>>> ")
        else:
            d = raw_input("Enter something\n>>> ")
        sys.stderr.write('This message has been written to stderr\n')
        print("You've just typed '%s' in the QTextEdit" % d)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#The MIT License (MIT)
#
#Copyright (c) <2013-2014> <Colin Duquesnoy and others, see AUTHORS.txt>
#
#Permission is hereby granted, free of charge, to any person obtaining a copy
#of this software and associated documentation files (the "Software"), to deal
#in the Software without restriction, including without limitation the rights
#to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
#copies of the Software, and to permit persons to whom the Software is
#furnished to do so, subject to the following conditions:
#
#The above copyright notice and this permission notice shall be included in
#all copies or substantial portions of the Software.
#
#THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
#IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
#FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
#AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
#LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
#OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
#THE SOFTWARE.
#
"""
Contains a widget to show and edit a PropertyRegistry
"""
import pyqode.core
from pyqode.qt import QtGui, QtCore

try:
    from pyqode.qt.QtCore import Property
except:
    class Property(object):
        def __init__(self, *args, **kwargs):
            pass

class QColorButton(QtGui.QWidget):
    """
    Simple color button used in :class:`pyqode.widgets.QPropertyGrid`.

    The widget is a combination of QPushButton and a QLabel layout with a
    QHBoxLayout.
    """

    #: Signal emitted when the color value changed.
    valueChanged = QtCore.Signal(QtGui.QColor)

    def __getColor(self):
        return self._color

    def __setColor(self, color):
        self._color = color
        self.label.setText(color.name())
        self.button.setStyleSheet("background-color: %s;" % color.name())

    #: The color to display
    color = Property(QtGui.QColor, fget=__getColor, fset=__setColor)

    def __init__(self, parent=None):
        QtGui.QWidget.__init__(self, parent)
        layout = QtGui.QHBoxLayout()
        layout.setContentsMargins(0, 0, 0, 0)

        self.button = QtGui.QToolButton()
        self.button.clicked.connect(self.showColorDialog)
        self._color = QtGui.QColor(255, 255, 255, 255)
        layout.addWidget(self.button)

        self.label = QtGui.QLabel()
        layout.addWidget(self.label)
        layout.setStretch(1, 1)
        self.setLayout(layout)

        self.color = QtGui.QColor("#FFFFFF")

    def showColorDialog(self):
        """
        Slots that executes a QColorDialog when the user click the on the
        button.
        """
        c = QtGui.QColorDialog.getColor(self._color, self)
        if c.isValid():
            self.__setColor(c)
            self.valueChanged.emit(c)


class QTextStyleWidget(QtGui.QWidget):
    """
    Widgets specialised to edit a :class:`pyqode.core.TextStyle`.
    """
    #: Signal emitted when the value changed.
    valueChanged = QtCore.Signal(object)

    @property
    def textStyle(self):
        """
        Gets/Sets the text style to edit.

        You retrieve the user modifications through this property.
        """
        return self.__textStyle

    @textStyle.setter
    def textStyle(self, textStyle):
        assert isinstance(textStyle, pyqode.core.TextStyle)
        self.colorButton.__setColor(textStyle.color)
        self.checkBoxBold.setChecked(textStyle.bold)
        self.checkBoxItalic.setChecked(textStyle.italic)
        self.checkBoxUnderlined.setChecked(textStyle.underlined)
        self.__textStyle = textStyle

    def __init__(self, parent=None):
        QtGui.QWidget.__init__(self, parent)
        layout = QtGui.QHBoxLayout()

        self.colorButton = QColorButton()
        self.colorButton.valueChanged.connect(self.onValueChanged)
        self.colorButton.label.hide()
        layout.addWidget(self.colorButton)

        self.checkBoxBold = QtGui.QCheckBox()
        self.checkBoxBold.stateChanged.connect(self.onValueChanged)
        self.checkBoxBold.setText("Bold")
        layout.addWidget(self.checkBoxBold)

        self.checkBoxItalic = QtGui.QCheckBox()
        self.checkBoxItalic.stateChanged.connect(self.onValueChanged)
        self.checkBoxItalic.setText("Italic")
        layout.addWidget(self.checkBoxItalic)

        self.checkBoxUnderlined = QtGui.QCheckBox()
        self.checkBoxUnderlined.stateChanged.connect(self.onValueChanged)
        self.checkBoxUnderlined.setText("Underlined")
        layout.addWidget(self.checkBoxUnderlined)
        layout.setContentsMargins(0, 0, 0, 0)
        layout.setStretch(3, 3)
        self.setLayout(layout)
        self.setContentsMargins(0, 0, 0, 0)

        self.__textStyle = None

    def onValueChanged(self):
        if self.__textStyle:
            ts = pyqode.core.TextStyle()
            ts.bold = self.checkBoxBold.isChecked()
            ts.italic = self.checkBoxItalic.isChecked()
            ts.underlined = self.checkBoxUnderlined.isChecked()
            ts.color = self.colorButton.color
            #if str(ts) != str(self.__textStyle):
            self.__textStyle = ts
            self.valueChanged.emit(ts)


class QPropertyGrid(QtGui.QWidget):
    """
    This widget displays and lets the user edit the properties of a
    :class:`pyqode.core.PropertyRegistry`

    Use :meth:`pyqode.widgets.setPropertyRegistry` set the property
    registry do edit. Use :attr:`pyqode.widgets.QPropertyGrid.registry` to get
    the modified property registry.

    You can use :attr:`pyqode.core.PropertyRegistry.valueChanged` to
    get notified about value changes.
    """

    rehighlightRequested = QtCore.Signal()

    @property
    def currentSection(self):
        """
        Gets the current section of the :class:`pyqode.core.PropetyRegistry`

        :rtype: str
        """
        return self.comboSections.currentText()

    def __init__(self, parent=None):
        QtGui.QWidget.__init__(self, parent)
        layout = QtGui.QVBoxLayout()
        #: The edited property registry.
        self.registry = None
        self.__widgets = {}

        # sections
        glayout = QtGui.QGridLayout()
        self.lblSection = QtGui.QLabel("Section")
        glayout.addWidget(self.lblSection, 0, 0)
        self.comboSections = QtGui.QComboBox()
        self.comboSections.currentIndexChanged.connect(self.onSectionChanged)
        glayout.addWidget(self.comboSections, 0, 1)
        glayout.setColumnStretch(1, 1)
        layout.addLayout(glayout)

        # grid
        self.table = QtGui.QTableWidget()
        hv = self.table.horizontalHeader()
        hv.setStretchLastSection(True)
        hv.setResizeMode(QtGui.QHeaderView.ResizeToContents)
        self.table.verticalHeader().hide()
        self.table.setHorizontalHeader(hv)
        #self.table.setMinimumSize(400, 0)
        self.table.setAlternatingRowColors(True)

        layout.addWidget(self.table)
        self.setLayout(layout)

    def setPropertyRegistry(self, registry):
        """
        Sets the :class:`pyqode.core.PropetyRegistry` to show/edit.

        :param registry: The property registry to edit/display
        :type registry: pyqode.core.PropertyRegistry
        """
        assert isinstance(registry, pyqode.core.PropertyRegistry)
        self.registry = None
        index = 0
        for i, name in enumerate(registry.sections()):
            if name == "General":
                index = i
            self.comboSections.addItem(name)
        self.comboSections.setCurrentIndex(index)
        section = self.comboSections.currentText()
        self._displayProperties(section, registry)
        self.registry = registry
        self.lblSection.setVisible(self.comboSections.count() > 1)
        self.comboSections.setVisible(self.comboSections.count() > 1)

    def onSectionChanged(self, index):
        if self.registry:
            section = self.comboSections.itemText(index)
            self._displayProperties(section, self.registry)

    def _displayProperties(self, section, registry):
        properties = registry.properties(section)
        self.__widgets.clear()
        self.table.clear()
        self.table.setColumnCount(2)
        self.table.setRowCount(0)
        self.table.setHorizontalHeaderLabels(["Property", "Value"])
        for k in sorted(properties.keys()):
            v = registry.value(k, section=section)
            if not isinstance(v, list):  # what to do with list items???
                row = self.table.rowCount()
                self.table.insertRow(row)
                # name item
                #item = QtGui.QTableWidgetItem(k)
                lbl = QtGui.QLabel(k)
                lbl.setContentsMargins(3, 0, 0, 0)
                #self.table.setItem(row, 0, item)
                self.table.setCellWidget(row, 0, lbl)
                widget = self.makePropertyItem(k, v)
                if widget:
                    widget.key = k
                    widget.setContentsMargins(3, 0, 0, 0)
                    self.table.setCellWidget(row, 1, widget)
                    self.__widgets[k] = widget
        hv = self.table.horizontalHeader()
        hv.adjustSize()

    def makePropertyItem(self, key, value):
        if key == "pygmentsStyle":  # specific case this is a combo box
            widget = QtGui.QComboBox()
            index = -1
            for i, s in enumerate(pyqode.core.PYGMENTS_STYLES):
                if s == value:
                    index = i
                widget.addItem(s)
            widget.setCurrentIndex(index)
            widget.currentIndexChanged.connect(self.onComboIndexChanged)
        elif key == "font":
            widget = QtGui.QFontComboBox()
            widget.setCurrentFont(QtGui.QFont(value))
            widget.currentFontChanged.connect(
                self.onFontComboBoxCurrentFontChanged)
            widget.setFontFilters(QtGui.QFontComboBox.MonospacedFonts)
            return widget
        elif isinstance(value, bool):
            widget = QtGui.QCheckBox()
            widget.setChecked(value)
            widget.stateChanged.connect(self.onCheckBoxStateChanged)
        elif isinstance(value, float):
            widget = QtGui.QDoubleSpinBox()
            widget.setValue(value)
            widget.editingFinished.connect(self.onSpinBoxEditingFinished)
        elif isinstance(value, int):
            widget = QtGui.QSpinBox()
            widget.setValue(value)
            widget.editingFinished.connect(self.onSpinBoxEditingFinished)
        elif isinstance(value, pyqode.core.TextStyle):
            widget = QTextStyleWidget()
            widget.textStyle = value
            widget.valueChanged.connect(self.onTextStyleValueChanged)
            return widget
        elif isinstance(value, QtGui.QColor):
            widget = QColorButton()
            widget.color = value
            widget.valueChanged.connect(self.onColorButtonValudeChanged)
            return widget
        else:
            widget = QtGui.QLineEdit()
            widget.setText(value)
            widget.editingFinished.connect(self.onLineEditEditingFinished)
        return widget

    def onComboIndexChanged(self):
        key = self.sender().key
        value = self.sender().currentText()
        self.registry.setValue(key, value, self.currentSection)
        if key == "pygmentsStyle":
            # the background, the foreground and the caret line background
            # properties must be updated
            k = "background"
            colorButton = self.__widgets[k]
            colorButton.color = self.registry.value(k, self.currentSection)
            colorButton.valueChanged.emit(colorButton.color)

            k = "foreground"
            colorButton = self.__widgets[k]
            colorButton.color = self.registry.value(k, self.currentSection)
            colorButton.valueChanged.emit(colorButton.color)

            k = "caretLineBackground"
            colorButton = self.__widgets[k]
            colorButton.color = self.registry.value(k, self.currentSection)
            colorButton.valueChanged.emit(colorButton.color)

    def onCheckBoxStateChanged(self, state):
        key = self.sender().key
        self.registry.setValue(key, bool(state), self.currentSection)

    def onSpinBoxEditingFinished(self):
        key = self.sender().key
        value = self.sender().value()
        self.registry.setValue(key, value, self.currentSection)

    def onLineEditEditingFinished(self):
        key = self.sender().key
        value = self.sender().text()
        self.registry.setValue(key, value, self.currentSection)

    def onColorButtonValudeChanged(self, value):
        key = self.sender().key
        self.registry.setValue(key, value, self.currentSection)

    def onFontComboBoxCurrentFontChanged(self, font):
        key = self.sender().key
        value = font.family()
        self.registry.setValue(key, value, self.currentSection)

    def onTextStyleValueChanged(self, value):
        key = self.sender().key
        self.registry.setValue(key, value, self.currentSection)
        self.rehighlightRequested.emit()


if __name__ == "__main__":
    import sys
    #import pyqode.python

    app = QtGui.QApplication(sys.argv)
    win = QtGui.QMainWindow()
    #editor = pyqode.python.QPythonCodeEdit()
    editor = pyqode.core.QGenericCodeEdit()
    editor.openFile(__file__, encoding="utf-8")
    win.setCentralWidget(editor)

    dockStyle = QtGui.QDockWidget("Style")
    grid = QPropertyGrid()
    grid.rehighlightRequested.connect(editor.syntaxHighlighterMode.rehighlight)
    grid.setPropertyRegistry(editor.style)
    grid.showNormal()
    dockStyle.setWidget(grid)
    win.addDockWidget(QtCore.Qt.RightDockWidgetArea, dockStyle)

    dockStyle = QtGui.QDockWidget("Settings")
    grid = QPropertyGrid()
    grid.setPropertyRegistry(editor.settings)
    grid.showNormal()
    dockStyle.setWidget(grid)
    win.addDockWidget(QtCore.Qt.RightDockWidgetArea, dockStyle)

    win.resize(1280, 1024)
    win.show()

    app.exec_()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#The MIT License (MIT)
#
#Copyright (c) <2013-2014> <Colin Duquesnoy and others, see AUTHORS.txt>
#
#Permission is hereby granted, free of charge, to any person obtaining a copy
#of this software and associated documentation files (the "Software"), to deal
#in the Software without restriction, including without limitation the rights
#to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
#copies of the Software, and to permit persons to whom the Software is
#furnished to do so, subject to the following conditions:
#
#The above copyright notice and this permission notice shall be included in
#all copies or substantial portions of the Software.
#
#THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
#IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
#FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
#AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
#LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
#OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
#THE SOFTWARE.
#
import os
import pyqode.qt


def loadUi(uiFileName, baseInstance, rcFilename=None):
    """
    Loads an ui file from the ui package and load the specified qrc (must
    already be compiled)

    :param uiFileName: The ui file name (without path)

    :param baseInstance: The baseInstance on which the ui is built

    :param rcFilename: The optional qrc file to load
    """
    uiFile = os.path.join(os.path.abspath(os.path.join(__file__, "..")),
                          uiFileName)
    if rcFilename:
        rcFile = os.path.join(os.path.abspath(os.path.join(__file__, "..")),
                              rcFilename)
        pyqode.qt.importRc(rcFile)
    pyqode.qt.loadUi(uiFile, baseInstance)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#The MIT License (MIT)
#
#Copyright (c) <2013-2014> <Colin Duquesnoy and others, see AUTHORS.txt>
#
#Permission is hereby granted, free of charge, to any person obtaining a copy
#of this software and associated documentation files (the "Software"), to deal
#in the Software without restriction, including without limitation the rights
#to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
#copies of the Software, and to permit persons to whom the Software is
#furnished to do so, subject to the following conditions:
#
#The above copyright notice and this permission notice shall be included in
#all copies or substantial portions of the Software.
#
#THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
#IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
#FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
#AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
#LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
#OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
#THE SOFTWARE.
#
"""
This module contains the implementation of a home widget suitable for IDE to
show the recent project list and the quick start action they propose.

This widget is based on https://github.com/ColinDuquesnoy/QWelcomeWindow
"""
import os
import sys
from pyqode.qt import QtGui, QtCore
from pyqode.widgets.ui.home_ui import Ui_Form


# needed to build doc on readthedocs; should be fixed with the next version
# of pyqode.core
try:
    from pyqode.qt.QtCore import Property, QPoint
    from pyqode.qt.QtGui import QIcon, QListWidgetItem
except:
    class Property(object):
        def __init__(self, *args, **kwargs):
            pass

    class QPoint(object):
        pass

    class QIcon(object):
        def __init__(self, *args, **kwargs):
            pass

    class QListWidgetItem(object):
        pass


class ColorScheme(object):
    """
    Define a color scheme for easy customisation of the QWelcomeWidget's
    stylesheet.

    There are 7 colors to define:
        - background color
        - title background color
        - text_color
        - border_color:
        - selection_background_color
        - selection_color
        - hover_color: The hover background color

    Color are defined as string with web/hex format: "#rrggbbb"

    The default colors define a white/light gray color scheme.
    """

    def __init__(self):
        #: widgets background
        self.background_color = "#FFFFFF"
        #: titles background (app title, recent title, quick start action title)
        if sys.platform == "win32":
            self.title_background_color = "#EEEEEE"
        else:
            self.title_background_color = "#DDDDDD"
        #: texts color
        self.text_color = "#555555"
        # frames border color
        self.border_color = "#cccccc"
        # item selection background color
        self.selection_bck_color = "#aaaaaa"
        #: selected item text color
        self.selection_color = "#FFFFFF"
        #: selected item hover background color
        self.hover_color = "#bbbbbb"


class DarkColorScheme(ColorScheme):
    """
    Define a dark color scheme for easy customization of the stylesheet.
    """

    def __init__(self):
        super(DarkColorScheme, self).__init__()
        self.background_color = "#3C3F41"
        self.title_background_color = "#4b4b4b"
        self.text_color = "#C5C5C5"
        self.border_color = "#555555"
        self.selection_bck_color = "#343434"
        self.selection_color = "#C5C5C5"
        self.hover_color = "#343434"


STYLESHEET = """
QWidget
{
    background-color: %(background_color)s;
    color: %(text_color)s;
    selection-background-color: %(selection_bck_color)s;
    selection-color: %(selection_color)s;
}

QFrame
{
    border: 1px solid %(border_color)s;
    background-color: %(background_color)s;
}

QListWidget
{
    border: none;
}

QListView::item
{
    padding: 5px;
}

QListView::item:selected
{
    border: 1px solid %(border_color)s;
}

QListView::item:selected:!active
{
    background: %(selection_bck_color)s;
    color: %(selection_color)s;
}

QListView::item:selected:active
{
    background: %(selection_bck_color)s;
    color: %(selection_color)s;
}

QListView::item:hover
{
    background: %(hover_color)s;
}

QLabel
{
    padding-top: 8px;
    padding-bottom: 8px;
    padding-left: 3px;
    padding-right: 0px;
    border: none;
    background-color: %(title_background_color)s;
};
"""


class QHomeWidget(QtGui.QWidget, Ui_Form):
    """
    A QWidget for displaying a home page. It lets you add your custom quick
    start actions and manage the list of recent files for you (and store them
    in the QSettings you specified).

    The only things you need to do is to is to call setup
    :meth:`pyqode.widgets.QHomeWidget.setupRecentFiles` to setup the recent file
    menu (and select the QSettings to use to store the list of recent files).

    You can also set :attr:`pyqode.widgets.QHomeWidget.title` and
    :attr:`pyqode.widgets.QHomeWidget.icon`

    Then you must call :meth:`pyqode.widgets.QHomeWidget.setCurrentFile`
    everytime you open a file and connect to
    :attr:`pyqode.widgets.QHomeWidget.fileOpenRequested` to know when the user
    wants to open a recent file.

    To add a quick start actions, you just have to call
    :meth:`pyqode.widgets.QHomeWidget.addAction`.
    """
    #: Defines the maximum number of recent files memorized in the app settings
    MaxRecentFiles = 10

    #: Signal emitted when a recent file action is triggered (from you QMenu
    #: or from the widget recent file list
    fileOpenRequested = QtCore.Signal(object)

    def __getTitle(self):
        return self.lblTitle.text()

    def __setTitle(self, value):
        self.lblTitle.setText(value)
        self.__appName = value

    #: Text displayed on the title zone
    title = Property(str, __getTitle, __setTitle)

    def __getIcon(self):
        return self.app_icon

    def __setIcon(self, value):
        self.setApplicationIcon(value)

    #: Icon displayed on the tile zone
    icon = Property(QIcon, __getIcon, __setIcon)

    def __init__(self,  parent=None, appName="YouAppName",
                 appIcon=None, colorScheme=None):
        """
        :param appName: Your application name. Displayed in the homepage and
                        used to open the application settings.

        :param appIcon: Your application icon.

        :param colorScheme: A color scheme to easily customize the widget's
                             stylesheet

        :param menuRecentFiles: Your recent files menu.

        :param actionClearMnuRecentFiles: The optional action used to clear the
                                          menu.
        """
        QtGui.QWidget.__init__(self, parent)
        Ui_Form.__init__(self)
        self.setupUi(self)
        self.setAutoFillBackground(True)
        self.__actions = []
        self.__appName = appName
        self.setColorScheme(colorScheme)
        self.app_icon = appIcon
        self.setApplicationIcon(appIcon)
        self.title %= appName
        # setup context menu
        self.lwRecents.setContextMenuPolicy(QtCore.Qt.CustomContextMenu)
        self.lwRecents.customContextMenuRequested.connect(
            self.__handleContextMenu)
        self.__contextMenu = QtGui.QMenu("Context Menu", self.lwRecents)
        self.__actionRemoveRecentFile = QtGui.QAction(
            "Remove", self)
        self.__actionRemoveRecentFile.triggered.connect(
            self.__removeCurrentFile)
        self.actionClearMnuRecentFiles = None

    @property
    def recentFiles(self):
        """
        Returns the list of recent files
        :return: a list of strings
        """
        ret_val = []
        files = self.__settings().value('recentFileList', [])
        if files is None:
            files = []
        if sys.version_info[0] == 2:
            if isinstance(files, unicode):
                files = [files]
        elif sys.version_info[0] == 3:
            if isinstance(files, str):
                files = [files]
        for f in files:
            if os.path.exists(f):
                ret_val.append(f)
        return ret_val

    @recentFiles.setter
    def recentFiles(self, files):
        """
        Save the list of rece,t files
        :param files: list of strings
        """
        self.__settings().setValue('recentFileList', files)

    def clearRecentFiles(self):
        """
        Clear the list of recent files
        :return:
        """
        self.recentFiles = []
        self.lwRecents.clear()
        self.__updateRecentFileActions()

    def setApplicationIcon(self, app_icon):
        if app_icon:
            self.lblIcon.setPixmap(app_icon.pixmap(32, 32))

    def changeApplicationLabel(self, app_name):
        self.lblTitle.setText("Welcome to %s" % app_name)

    def setColorScheme(self, color_scheme):
        """
        Sets the widget color scheme

        :param color_scheme: Color scheme instance
        :type color_scheme: pyqode.widgets.ColorScheme
        """
        if color_scheme is None:
            color_scheme = ColorScheme()
        stylesheet = STYLESHEET % {
            "background_color": color_scheme.background_color,
            "title_background_color": color_scheme.title_background_color,
            "text_color": color_scheme.text_color,
            "border_color": color_scheme.border_color,
            "selection_bck_color": color_scheme.selection_bck_color,
            "selection_color": color_scheme.selection_color,
            "hover_color": color_scheme.hover_color}
        self.setStyleSheet(stylesheet)
        p = self.palette()
        p.window().setColor(QtGui.QColor(color_scheme.background_color))
        self.setPalette(p)

    def addAction(self, action):
        """
        Adds a quick start action.

        :param action: QAction
        """
        assert isinstance(action, QtGui.QAction)
        self.__actions.append(action)
        item = QtGui.QListWidgetItem(action.text())
        item.setIcon(action.icon())
        self.lwQuickStart.addItem(item)
        QtGui.QWidget.addAction(self, action)

    def setActionIcon(self, action, icon):
        for i in range(self.lwQuickStart.count()):
            item = self.lwQuickStart.item(i)
            if item.text() == action.text():
                item.setIcon(action.icon())
                break

    def setCurrentFile(self, fileName):
        """
        Set the current filename (move it to the top of the stack of recent
        files)
        """
        self.curFile = fileName
        files = self.recentFiles
        try:
            files.remove(fileName)
        except ValueError:
            pass
        files.insert(0, fileName)
        del files[self.MaxRecentFiles:]
        self.recentFiles = files
        self.__updateRecentFileActions()

    def setActionText(self, index, text):
        """
        Changes visible text of a specific quick start action.
        """
        item = self.lwQuickStart.item(index)
        if not item:
            raise IndexError(index)
        item.setText(text)
        self.__actions[index].setText(text)

    def setupRecentFiles(self, organization="YourOrg", app="YourAppName",
                         menuRecentFiles=None, actionClearMnuRecentFiles=None):
        """
        Loads the recent file list, create its recent actions and create the
        recent files menu.
        """
        self.__org = organization
        self.__app = app
        self.actionClearMnuRecentFiles = actionClearMnuRecentFiles
        if actionClearMnuRecentFiles:
            actionClearMnuRecentFiles.triggered.connect(self.clearRecentFiles)
        self.menuRecentFiles = menuRecentFiles
        self.actionClearMnuRecentFiles = actionClearMnuRecentFiles
        self.separatorAct = None

        self.recentFileActs = []
        self.__createActions()
        self.__createMenus()

    def __addRecentFileAction(self, filePath):
        fileName = QtCore.QFileInfo(filePath).fileName()
        item = QtGui.QListWidgetItem(fileName)
        item.setData(32, filePath)
        self.lwRecents.addItem(item)

    def __settings(self):
        return QtCore.QSettings(self.__org, self.__app)

    def __createActions(self):
        """
        Create recent file actions
        """
        for i in range(self.MaxRecentFiles):
            self.recentFileActs.append(
                QtGui.QAction(self, visible=False,
                              triggered=self.__onOpenRecentFile))

    def __onOpenRecentFile(self):
        """
        Slot called when a recent file action is triggered, emits the
        recent_action_triggered.
        """
        action = self.sender()
        if action:
            path = action.data()
            if os.path.exists(path):
                self.fileOpenRequested.emit(path)

    def __createMenus(self):
        """
        Create menus
        """
        if self.menuRecentFiles:
            self.menuRecentFiles.clear()
            for i in range(self.MaxRecentFiles):
                self.menuRecentFiles.addAction(self.recentFileActs[i])
            self.separatorAct = self.menuRecentFiles.addSeparator()
            self.menuRecentFiles.addAction(self.actionClearMnuRecentFiles)
        self.__updateRecentFileActions()

    def __updateRecentFileActions(self):
        """
        Update the recent file actions.

        Clears the menu and the recent actions, then fill them with the list
        stored in the app settings
        """
        files = self.recentFiles
        files_no = 0
        if files:
            files_no = len(files)
        numRecentFiles = min(files_no, self.MaxRecentFiles)
        self.lwRecents.clear()
        for i in range(numRecentFiles):
            text = "%s" % self.__strippedName(files[i])
            self.recentFileActs[i].setText(text)
            self.recentFileActs[i].setData(files[i])
            self.recentFileActs[i].setVisible(True)
            self.__addRecentFileAction(files[i])
        for j in range(numRecentFiles, self.MaxRecentFiles):
            self.recentFileActs[j].setVisible(False)
        if self.separatorAct:
            self.separatorAct.setVisible((numRecentFiles > 0))

    def __strippedName(self, fullFileName):
        """
        Returs  the stripped filename (name + extension)

        :param fullFileName: The full filename to strip

        :return: The stripped name
        """
        return QtCore.QFileInfo(fullFileName).fileName()

    def __removeCurrentFile(self):
        fn = self.lwRecents.currentItem().data(32)
        files = self.recentFiles
        try:
            files.remove(fn)
        except ValueError:
            pass
        del files[self.MaxRecentFiles:]
        self.recentFiles = files
        self.__updateRecentFileActions()

    @QtCore.Slot(QListWidgetItem)
    def on_lwRecents_itemClicked(self, item=None):
        if self.lwRecents.mouseButton == QtCore.Qt.LeftButton:
            if item is None:
                item = self.lwRecents.currentItem()
            self.lwRecents.clearSelection()
            path = item.data(32)
            if os.path.exists(path):
                self.fileOpenRequested.emit(path)
            else:
                QtGui.QMessageBox.information(
                    self.parent(), "File does not exits",
                    "The file %s does not exists, it will be removed from the "
                    "recent files list" % path)
                self.__updateRecentFileActions()

    @QtCore.Slot(QListWidgetItem)
    def on_lwQuickStart_itemClicked(self, item):
        if self.lwQuickStart.mouseButton == QtCore.Qt.LeftButton:
            self.lwQuickStart.clearSelection()
            for action in self.__actions:
                if action.text() == item.text():
                    if os.environ['QT_API'] == "PySide":
                        action.triggered.emit()
                    else:
                        action.triggered.emit(True)

    @QtCore.Slot(QPoint)
    def __handleContextMenu(self, pos):
        self.__contextMenu.clear()
        item = self.lwRecents.itemAt(pos)
        if item is not None:
            self.__contextMenu.addAction(self.__actionRemoveRecentFile)
        if self.actionClearMnuRecentFiles:
            self.__contextMenu.addAction(self.actionClearMnuRecentFiles)
        self.__contextMenu.exec_(self.lwRecents.mapToGlobal(pos))


#///////////////////////////////////////////////////////////////////////////////
# Example
#///////////////////////////////////////////////////////////////////////////////
_widget = None
_theme = None


def onChangeTheme():
    global _widget, _theme
    if _theme is None:
        _theme = DarkColorScheme()
        _widget.setColorScheme(_theme)
        _widget.setActionText(2, "Try white theme")
    else:
        _widget.setColorScheme(ColorScheme())
        _theme = None
        _widget.setActionText(2, "Try dark theme")


def showHelp():
    QtGui.QMessageBox.information(
        None, "QWelcomeWindow",
        "QHomeWidget is a simple widget that you can use to quickly "
        "create a home window for your PySide/PyQt4 application.")


def openFile():
    fn = QtGui.QFileDialog.getOpenFileName(None)
    if fn:
        global _widget
        if os.environ['QT_API'] == "PySide":
            _widget.setCurrentFile(fn[0])
        else:
            _widget.setCurrentFile(fn)


def onOpenRecentFileRequested(filePath):
    print("User requested to open %s" % filePath)


def main():
    """
    Simple example
    """
    global _widget


    app = QtGui.QApplication(sys.argv)
    style = app.style()

    # create the widget
    _widget = QHomeWidget(appName="YourAppName",
                          appIcon=style.standardIcon(style.SP_ComputerIcon))

    # setup recent files actions
    actionClearRecents = QtGui.QAction("Clear", _widget)
    actionClearRecents.triggered.connect(_widget.clearRecentFiles)
    _widget.setupRecentFiles(organization="YourOrg",
                             actionClearMnuRecentFiles=actionClearRecents)
    _widget.fileOpenRequested.connect(onOpenRecentFileRequested)

    # Quick start actions
    openAction = QtGui.QAction(
        style.standardIcon(style.SP_DialogOpenButton), "Open", _widget)
    openAction.triggered.connect(openFile)
    _widget.addAction(openAction)

    helpAction = QtGui.QAction(
        style.standardIcon(style.SP_DialogHelpButton), "Help", _widget)
    helpAction.triggered.connect(showHelp)
    _widget.addAction(helpAction)

    reloadAction = QtGui.QAction(
        style.standardIcon(style.SP_BrowserReload), "Try dark theme", _widget)
    reloadAction.triggered.connect(onChangeTheme)
    _widget.addAction(reloadAction)

    # run the gui app
    _widget.show()
    app.exec_()


if __name__ == "__main__":
    main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#The MIT License (MIT)
#
#Copyright (c) <2013-2014> <Colin Duquesnoy and others, see AUTHORS.txt>
#
#Permission is hereby granted, free of charge, to any person obtaining a copy
#of this software and associated documentation files (the "Software"), to deal
#in the Software without restriction, including without limitation the rights
#to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
#copies of the Software, and to permit persons to whom the Software is
#furnished to do so, subject to the following conditions:
#
#The above copyright notice and this permission notice shall be included in
#all copies or substantial portions of the Software.
#
#THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
#IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
#FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
#AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
#LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
#OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
#THE SOFTWARE.
#
"""
This setup script packages pyqode.widgets
"""
from setuptools import setup, find_packages


def read_version():
    with open("pyqode/widgets/__init__.py") as f:
        lines = f.read().splitlines()
        for l in lines:
            if "__version__" in l:
                return l.split("=")[1].strip().replace('"', '')


def readme():
    return str(open('README.rst').read())


# get requirements
requirements = ['pyqode.core>=1.3.1']


setup(
    name='pyqode.widgets',
    namespace_packages=['pyqode'],
    version=read_version(),
    packages=find_packages(),
    keywords=["pyQode", "PySide", "PyQt", "widgets"],
    package_dir={'pyqode': 'pyqode'},
    # package_data={'pyqode.widgets.ui': ['*.ui', '*.qrc', 'rc/*']},
    url='https://github.com/pyQode/pyqode.widgets',
    license='MIT',
    author='Colin Duquesnoy',
    author_email='colin.duquesnoy@gmail.com',
    description='A set of widgets for pyQode applications',
    long_description=readme(),
    install_requires=requirements,
    entry_points={'pyqode_plugins':
                 ['pyqode_widgets = pyqode.widgets.plugins.pyqode_widgets_plugin']},
    zip_safe=False,
    classifiers=[
          'Development Status :: 5 - Production/Stable',
          'Environment :: X11 Applications :: Qt',
          'Environment :: Win32 (MS Windows)',
          'Intended Audience :: Developers',
          'License :: OSI Approved :: MIT License',
          'Operating System :: Microsoft :: Windows',
          'Operating System :: POSIX :: Linux',
          'Programming Language :: Python :: 2.7',
          'Programming Language :: Python :: 3.2',
          'Programming Language :: Python :: 3.3',
          'Topic :: Software Development :: Libraries :: Application Frameworks',
          'Topic :: Software Development :: Widget Sets']
)




#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#The MIT License (MIT)
#
#Copyright (c) <2013-2014> <Colin Duquesnoy and others, see AUTHORS.txt>
#
#Permission is hereby granted, free of charge, to any person obtaining a copy
#of this software and associated documentation files (the "Software"), to deal
#in the Software without restriction, including without limitation the rights
#to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
#copies of the Software, and to permit persons to whom the Software is
#furnished to do so, subject to the following conditions:
#
#The above copyright notice and this permission notice shall be included in
#all copies or substantial portions of the Software.
#
#THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
#IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
#FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
#AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
#LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
#OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
#THE SOFTWARE.
#
"""
This scripts compile the ui and qrc files using pyside dev tools then modify
them to use pyqode.qt instead of PySide. It also adapts the rc imports so that
they works with python3
"""
import glob
import os


def fix_script(script):
    with open(script, 'r') as f_script:
        lines = f_script.read().splitlines()
    new_lines = []
    for l in lines:
        if l.startswith("import "):
            l = "from . " + l
        if "from PySide import" in l:
            l = l.replace("from PySide import", "from pyqode.qt import")
        new_lines.append(l)
    with open(script, 'w') as f_script:
        f_script.write("\n".join(new_lines))


def main():
    # ui scripts
    for ui_file in glob.glob("*.ui"):
        base_name = os.path.splitext(ui_file)[0]
        dst = "%s_ui.py" % base_name
        cmd = "pyside-uic %s -o %s" % (ui_file, dst)
        print(cmd)
        os.system(cmd)
        fix_script(dst)

    # rc scripts
    for rc_file in glob.glob("*.qrc"):
        base_name = os.path.splitext(rc_file)[0]
        dst = "%s_rc.py" % base_name
        cmd = "pyside-rcc -py3 %s -o %s" % (rc_file, dst)
        print(cmd)
        os.system(cmd)
        fix_script(dst)

if __name__ == "__main__":
    main()
